@@ -34,10 +34,10 @@ exe_generate = os.path.join(os.path.dirname(os.path.dirname(
 os.environ['G_DEBUG'] = 'fatal-criticals'
 
 # common patterns for expected output
-ND_DHCP4 = '[Match]\nName=%s\n\n[Network]\nDHCP=ipv4\n\n[DHCP]\nRouteMetric=100\n'
-ND_WIFI_DHCP4 = '[Match]\nName=%s\n\n[Network]\nDHCP=ipv4\n\n[DHCP]\nRouteMetric=600\n'
-ND_DHCP6 = '[Match]\nName=%s\n\n[Network]\nDHCP=ipv6\n\n[DHCP]\nRouteMetric=100\n'
-ND_DHCPYES = '[Match]\nName=%s\n\n[Network]\nDHCP=yes\n\n[DHCP]\nRouteMetric=100\n'
+ND_DHCP4 = '[Match]\nName=%s\n\n[Network]\nDHCP=ipv4\n\n[DHCP]\nUseMTU=true\nRouteMetric=100\n'
+ND_WIFI_DHCP4 = '[Match]\nName=%s\n\n[Network]\nDHCP=ipv4\n\n[DHCP]\nUseMTU=true\nRouteMetric=600\n'
+ND_DHCP6 = '[Match]\nName=%s\n\n[Network]\nDHCP=ipv6\n\n[DHCP]\nUseMTU=true\nRouteMetric=100\n'
+ND_DHCPYES = '[Match]\nName=%s\n\n[Network]\nDHCP=yes\n\n[DHCP]\nUseMTU=true\nRouteMetric=100\n'
 
 
 class TestBase(unittest.TestCase):
@@ -256,10 +256,38 @@ class TestConfigArgs(TestBase):
         # no enablement symlink here
         self.assertEqual(os.listdir(outdir), ['netplan.stamp'])
 
+    def test_systemd_generator_badcall(self):
+        outdir = os.path.join(self.workdir.name, 'out')
+        os.mkdir(outdir)
+
+        generator = os.path.join(self.workdir.name, 'systemd', 'system-generators', 'netplan')
+        os.makedirs(os.path.dirname(generator))
+        os.symlink(exe_generate, generator)
+
+        try:
+            subprocess.check_output([generator, '--root-dir', self.workdir.name],
+                                    stderr=subprocess.STDOUT)
+            self.fail("direct systemd generator call is expected to fail, but succeeded.")
+        except subprocess.CalledProcessError as e:
+            self.assertEqual(e.returncode, 1)
+            self.assertIn(b'can not be called directly', e.output)
+
 
 class TestNetworkd(TestBase):
     '''networkd output'''
 
+    def test_eth_optional(self):
+        # TODO: cyphermox: this is to validate that "optional" does not cause
+        #       any extra config to be generated; and will fail once it's actually
+        #       implemented.
+        self.generate('''network:
+  version: 2
+  ethernets:
+    eth0:
+      dhcp6: true
+      optional: true''')
+        self.assert_networkd({'eth0.network': ND_DHCP6 % 'eth0'})
+
     def test_eth_wol(self):
         self.generate('''network:
   version: 2
@@ -358,7 +386,16 @@ unmanaged-devices+=mac:11:22:33:44:55:66,''')
         driver: ixgbe
       dhcp4: true''')
 
-        self.assert_networkd({'def1.network': '[Match]\nDriver=ixgbe\n\n[Network]\nDHCP=ipv4\n\n[DHCP]\nRouteMetric=100\n'})
+        self.assert_networkd({'def1.network': '''[Match]
+Driver=ixgbe
+
+[Network]
+DHCP=ipv4
+
+[DHCP]
+UseMTU=true
+RouteMetric=100
+'''})
         self.assert_udev('ACTION=="add|change", SUBSYSTEM=="net", ENV{ID_NET_DRIVER}=="ixgbe", ENV{NM_UNMANAGED}="1"\n')
 
     def test_eth_match_name(self):
@@ -429,7 +466,8 @@ unmanaged-devices+=interface-name:*,''')
       match: {}
       dhcp4: true''')
 
-        self.assert_networkd({'def1.network': '[Match]\n\n[Network]\nDHCP=ipv4\n\n[DHCP]\nRouteMetric=100\n'})
+        self.assert_networkd({'def1.network': '[Match]\n\n[Network]\nDHCP=ipv4\n\n'
+                                              '[DHCP]\nUseMTU=true\nRouteMetric=100\n'})
         self.assert_nm(None, '''[keyfile]
 # devices managed by networkd
 unmanaged-devices+=type:ethernet,''')
@@ -452,6 +490,7 @@ Name=en1s*
 DHCP=ipv4
 
 [DHCP]
+UseMTU=true
 RouteMetric=100
 '''})
         self.assert_nm(None, '''[keyfile]
@@ -525,6 +564,25 @@ unmanaged-devices+=interface-name:eth0,''')
     eth0: {dhcp6: true}''')
         self.assert_networkd({'eth0.network': ND_DHCP6 % 'eth0'})
 
+    def test_eth_dhcp6_no_accept_ra(self):
+        self.generate('''network:
+  version: 2
+  ethernets:
+    eth0:
+      dhcp6: true
+      accept-ra: no''')
+        self.assert_networkd({'eth0.network': '''[Match]
+Name=eth0
+
+[Network]
+DHCP=ipv6
+IPv6AcceptRA=no
+
+[DHCP]
+UseMTU=true
+RouteMetric=100
+'''})
+
     def test_eth_dhcp4_and_6(self):
         self.generate('''network:
   version: 2
@@ -568,6 +626,7 @@ Address=192.168.14.2/24
 Address=2001:FFfe::1/64
 
 [DHCP]
+UseMTU=true
 RouteMetric=100
 '''})
 
@@ -749,6 +808,7 @@ Destination=10.10.10.0/24
 Gateway=8.8.8.8
 
 [DHCP]
+UseMTU=true
 RouteMetric=600
 '''})
 
@@ -831,7 +891,9 @@ Name=br0
 DHCP=ipv4
 Address=1.2.3.4/12
 
-[DHCP]\nRouteMetric=100
+[DHCP]
+UseMTU=true
+RouteMetric=100
 '''})
         self.assert_nm(None, '''[keyfile]
 # devices managed by networkd
@@ -859,6 +921,22 @@ unmanaged-devices+=interface-name:br0,''')
                               'switchports.network': '[Match]\nDriver=yayroute\n\n'
                                                      '[Network]\nBridge=br0\nLinkLocalAddressing=no\nIPv6AcceptRA=no\n'})
 
+    def test_eth_bridge_nm_blacklist(self):
+        self.generate('''network:
+  renderer: networkd
+  ethernets:
+    eth42:
+      dhcp4: yes
+    ethbr:
+      match: {name: eth43}
+  bridges:
+    mybr:
+      interfaces: [ethbr]
+      dhcp4: yes''')
+        self.assert_nm(None, '''[keyfile]
+# devices managed by networkd
+unmanaged-devices+=interface-name:eth42,interface-name:eth43,interface-name:mybr,''')
+
     def test_bridge_components(self):
         self.generate('''network:
   version: 2
@@ -1032,6 +1110,31 @@ unmanaged-devices+=interface-name:bn0,''')
                               'switchports.network': '[Match]\nDriver=yayroute\n\n'
                                                      '[Network]\nBond=bn0\nLinkLocalAddressing=no\nIPv6AcceptRA=no\n'})
 
+    def test_bond_primary_slave(self):
+        self.generate('''network:
+  version: 2
+  ethernets:
+    eno1: {}
+    switchports:
+      match:
+        driver: yayroute
+  bonds:
+    bn0:
+      parameters:
+        mode: active-backup
+        primary: eno1
+      interfaces: [eno1, switchports]
+      dhcp4: true''')
+
+        self.assert_networkd({'bn0.netdev': '[NetDev]\nName=bn0\nKind=bond\n\n'
+                                            '[Bond]\n'
+                                            'Mode=active-backup\n',
+                              'bn0.network': ND_DHCP4 % 'bn0',
+                              'eno1.network': '[Match]\nName=eno1\n\n'
+                                              '[Network]\nBond=bn0\nLinkLocalAddressing=no\nIPv6AcceptRA=no\nPrimarySlave=true\n',
+                              'switchports.network': '[Match]\nDriver=yayroute\n\n'
+                                                     '[Network]\nBond=bn0\nLinkLocalAddressing=no\nIPv6AcceptRA=no\n'})
+
     def test_gateway(self):
         self.generate('''network:
   version: 2
@@ -1129,12 +1232,112 @@ wake-on-lan=1
 
 [ipv4]
 method=link-local
+
+[ipv6]
+method=ignore
 '''})
         # should allow NM to manage everything else
         self.assertTrue(os.path.exists(self.nm_enable_all_conf))
         self.assert_networkd({'eth0.link': '[Match]\nOriginalName=eth0\n\n[Link]\nWakeOnLan=magic\n'})
         self.assert_udev(None)
 
+    def test_eth_mtu(self):
+        self.generate('''network:
+  version: 2
+  renderer: NetworkManager
+  ethernets:
+    eth1:
+      mtu: 1280
+      dhcp4: n''')
+
+        self.assert_networkd({'eth1.link': '[Match]\nOriginalName=eth1\n\n[Link]\nWakeOnLan=off\nMTUBytes=1280\n'})
+        self.assert_nm({'eth1': '''[connection]
+id=netplan-eth1
+type=ethernet
+interface-name=eth1
+
+[ethernet]
+wake-on-lan=0
+
+[802-3-ethernet]
+mtu=1280
+
+[ipv4]
+method=link-local
+
+[ipv6]
+method=ignore
+'''})
+
+    def test_mtu_all(self):
+        self.generate(textwrap.dedent("""
+            network:
+              version: 2
+              renderer: NetworkManager
+              ethernets:
+                eth1:
+                  mtu: 1280
+                  dhcp4: n
+              bonds:
+                bond0:
+                  interfaces:
+                  - eth1
+                  mtu: 9000
+              vlans:
+                bond0.108:
+                  link: bond0
+                  id: 108"""))
+        self.assert_nm({
+            'bond0.108': '''[connection]
+id=netplan-bond0.108
+type=vlan
+interface-name=bond0.108
+
+[vlan]
+id=108
+parent=bond0
+
+[ipv4]
+method=link-local
+
+[ipv6]
+method=ignore
+''',
+            'bond0': '''[connection]
+id=netplan-bond0
+type=bond
+interface-name=bond0
+
+[802-3-ethernet]
+mtu=9000
+
+[ipv4]
+method=link-local
+
+[ipv6]
+method=ignore
+''',
+            'eth1': '''[connection]
+id=netplan-eth1
+type=ethernet
+interface-name=eth1
+slave-type=bond
+master=bond0
+
+[ethernet]
+wake-on-lan=0
+
+[802-3-ethernet]
+mtu=1280
+
+[ipv4]
+method=link-local
+
+[ipv6]
+method=ignore
+''',
+        })
+
     def test_eth_set_mac(self):
         self.generate('''network:
   version: 2
@@ -1165,6 +1368,9 @@ cloned-mac-address=00:01:02:03:04:05
 
 [ipv4]
 method=auto
+
+[ipv6]
+method=ignore
 '''})
 
     def test_eth_match_by_driver(self):
@@ -1199,6 +1405,9 @@ wake-on-lan=0
 
 [ipv4]
 method=link-local
+
+[ipv6]
+method=ignore
 '''})
         self.assert_udev(None)
 
@@ -1223,6 +1432,9 @@ wake-on-lan=0
 
 [ipv4]
 method=link-local
+
+[ipv6]
+method=ignore
 '''})
         self.assert_udev(None)
 
@@ -1244,6 +1456,9 @@ wake-on-lan=0
 
 [ipv4]
 method=auto
+
+[ipv6]
+method=ignore
 '''})
         self.assert_networkd({})
 
@@ -1269,6 +1484,9 @@ mac-address=11:22:33:44:55:66
 
 [ipv4]
 method=auto
+
+[ipv6]
+method=ignore
 '''})
         self.assert_networkd({})
 
@@ -1292,6 +1510,9 @@ wake-on-lan=0
 
 [ipv4]
 method=auto
+
+[ipv6]
+method=ignore
 '''})
         self.assert_networkd({})
         self.assert_udev(None)
@@ -1318,6 +1539,9 @@ wake-on-lan=0
 
 [ipv4]
 method=auto
+
+[ipv6]
+method=ignore
 '''})
         # ... while udev renames it
         self.assert_networkd({'def1.link': '[Match]\nOriginalName=green\n\n[Link]\nName=blue\nWakeOnLan=off\n'})
@@ -1354,6 +1578,9 @@ wake-on-lan=0
 
 [ipv4]
 method=auto
+
+[ipv6]
+method=ignore
 '''})
         self.assert_networkd({})
 
@@ -1380,6 +1607,9 @@ mac-address=00:11:22:33:44:55
 
 [ipv4]
 method=auto
+
+[ipv6]
+method=ignore
 '''})
         self.assert_networkd({})
         self.assert_udev(None)
@@ -1402,6 +1632,9 @@ wake-on-lan=0
 
 [ipv4]
 method=auto
+
+[ipv6]
+method=ignore
 '''})
         self.assert_networkd({})
         self.assert_udev(None)
@@ -1425,6 +1658,9 @@ wake-on-lan=0
 
 [ipv4]
 method=auto
+
+[ipv6]
+method=ignore
 '''})
         self.assert_networkd({})
         self.assert_udev(None)
@@ -1448,6 +1684,9 @@ wake-on-lan=0
 
 [ipv4]
 method=link-local
+
+[ipv6]
+method=ignore
 '''})
         self.assert_networkd({})
         self.assert_udev(None)
@@ -1587,6 +1826,9 @@ wake-on-lan=0
 method=manual
 address1=192.168.14.2/24
 route1=10.10.10.0/24,192.168.14.20,100
+
+[ipv6]
+method=ignore
 '''})
 
     def test_route_v4_multiple(self):
@@ -1621,6 +1863,9 @@ address1=192.168.14.2/24
 route1=8.8.0.0/16,192.168.1.1,5000
 route2=10.10.10.8,192.168.1.2
 route3=11.11.11.0/24,192.168.1.3,9999
+
+[ipv6]
+method=ignore
 '''})
 
     def test_route_v6_single(self):
@@ -1752,6 +1997,9 @@ wake-on-lan=0
 [ipv4]
 method=auto
 
+[ipv6]
+method=ignore
+
 [wifi]
 ssid=Joe's Home
 mode=infrastructure
@@ -1771,6 +2019,9 @@ wake-on-lan=0
 [ipv4]
 method=auto
 
+[ipv6]
+method=ignore
+
 [wifi]
 ssid=workplace
 mode=infrastructure
@@ -1806,6 +2057,9 @@ mac-address=11:22:33:44:55:66
 [ipv4]
 method=link-local
 
+[ipv6]
+method=ignore
+
 [wifi]
 ssid=workplace
 mode=infrastructure
@@ -1831,6 +2085,9 @@ wake-on-lan=0
 [ipv4]
 method=link-local
 
+[ipv6]
+method=ignore
+
 [wifi]
 ssid=workplace
 mode=infrastructure
@@ -1858,6 +2115,9 @@ wake-on-lan=0
 [ipv4]
 method=shared
 
+[ipv6]
+method=ignore
+
 [wifi]
 ssid=homenet
 mode=ap
@@ -1890,6 +2150,9 @@ wake-on-lan=0
 [ipv4]
 method=link-local
 
+[ipv6]
+method=ignore
+
 [wifi]
 ssid=homenet
 mode=adhoc
@@ -1910,6 +2173,9 @@ interface-name=br0
 
 [ipv4]
 method=auto
+
+[ipv6]
+method=ignore
 '''})
         self.assert_networkd({})
         self.assert_udev(None)
@@ -1930,6 +2196,9 @@ interface-name=br0
 
 [ipv4]
 method=auto
+
+[ipv6]
+method=ignore
 '''})
         self.assert_networkd({})
         self.assert_udev(None)
@@ -1953,6 +2222,9 @@ cloned-mac-address=00:01:02:03:04:05
 
 [ipv4]
 method=auto
+
+[ipv6]
+method=ignore
 '''})
 
     def test_bridge_def_renderer(self):
@@ -1974,6 +2246,9 @@ interface-name=br0
 [ipv4]
 method=auto
 address1=1.2.3.4/12
+
+[ipv6]
+method=ignore
 '''})
         self.assert_networkd({})
         self.assert_udev(None)
@@ -2005,6 +2280,9 @@ wake-on-lan=0
 
 [ipv4]
 method=link-local
+
+[ipv6]
+method=ignore
 ''',
                         'switchport': '''[connection]
 id=netplan-switchport
@@ -2018,6 +2296,9 @@ wake-on-lan=0
 
 [ipv4]
 method=link-local
+
+[ipv6]
+method=ignore
 ''',
                         'br0': '''[connection]
 id=netplan-br0
@@ -2026,6 +2307,9 @@ interface-name=br0
 
 [ipv4]
 method=auto
+
+[ipv6]
+method=ignore
 '''})
         self.assert_networkd({})
         self.assert_udev(None)
@@ -2056,6 +2340,9 @@ wake-on-lan=0
 
 [ipv4]
 method=link-local
+
+[ipv6]
+method=ignore
 ''',
                         'switchport': '''[connection]
 id=netplan-switchport
@@ -2069,6 +2356,9 @@ wake-on-lan=0
 
 [ipv4]
 method=link-local
+
+[ipv6]
+method=ignore
 ''',
                         'br0': '''[connection]
 id=netplan-br0
@@ -2077,6 +2367,9 @@ interface-name=br0
 
 [ipv4]
 method=auto
+
+[ipv6]
+method=ignore
 '''})
         self.assert_networkd({})
         self.assert_udev(None)
@@ -2119,6 +2412,9 @@ wake-on-lan=0
 
 [ipv4]
 method=link-local
+
+[ipv6]
+method=ignore
 ''',
                         'switchport': '''[connection]
 id=netplan-switchport
@@ -2132,6 +2428,9 @@ wake-on-lan=0
 
 [ipv4]
 method=link-local
+
+[ipv6]
+method=ignore
 ''',
                         'br0': '''[connection]
 id=netplan-br0
@@ -2148,6 +2447,9 @@ stp=true
 
 [ipv4]
 method=auto
+
+[ipv6]
+method=ignore
 '''})
         self.assert_networkd({})
         self.assert_udev(None)
@@ -2167,6 +2469,9 @@ interface-name=bn0
 
 [ipv4]
 method=auto
+
+[ipv6]
+method=ignore
 '''})
 
     def test_bond_components(self):
@@ -2195,6 +2500,9 @@ wake-on-lan=0
 
 [ipv4]
 method=link-local
+
+[ipv6]
+method=ignore
 ''',
                         'switchport': '''[connection]
 id=netplan-switchport
@@ -2208,6 +2516,9 @@ wake-on-lan=0
 
 [ipv4]
 method=link-local
+
+[ipv6]
+method=ignore
 ''',
                         'bn0': '''[connection]
 id=netplan-bn0
@@ -2216,6 +2527,9 @@ interface-name=bn0
 
 [ipv4]
 method=auto
+
+[ipv6]
+method=ignore
 '''})
         self.assert_networkd({})
         self.assert_udev(None)
@@ -2247,6 +2561,9 @@ wake-on-lan=0
 
 [ipv4]
 method=link-local
+
+[ipv6]
+method=ignore
 ''',
                         'switchport': '''[connection]
 id=netplan-switchport
@@ -2260,6 +2577,9 @@ wake-on-lan=0
 
 [ipv4]
 method=link-local
+
+[ipv6]
+method=ignore
 ''',
                         'bn0': '''[connection]
 id=netplan-bn0
@@ -2268,6 +2588,9 @@ interface-name=bn0
 
 [ipv4]
 method=auto
+
+[ipv6]
+method=ignore
 '''})
         self.assert_networkd({})
         self.assert_udev(None)
@@ -2320,6 +2643,9 @@ wake-on-lan=0
 
 [ipv4]
 method=link-local
+
+[ipv6]
+method=ignore
 ''',
                         'switchport': '''[connection]
 id=netplan-switchport
@@ -2333,6 +2659,9 @@ wake-on-lan=0
 
 [ipv4]
 method=link-local
+
+[ipv6]
+method=ignore
 ''',
                         'bn0': '''[connection]
 id=netplan-bn0
@@ -2362,6 +2691,76 @@ lp_interval=10
 
 [ipv4]
 method=auto
+
+[ipv6]
+method=ignore
+'''})
+        self.assert_networkd({})
+        self.assert_udev(None)
+
+    def test_bond_primary_slave(self):
+        self.generate('''network:
+  version: 2
+  renderer: NetworkManager
+  ethernets:
+    eno1: {}
+    switchport:
+      match:
+        name: enp2s1
+  bonds:
+    bn0:
+      interfaces: [eno1, switchport]
+      parameters:
+        mode: active-backup
+        primary: eno1
+      dhcp4: true''')
+
+        self.assert_nm({'eno1': '''[connection]
+id=netplan-eno1
+type=ethernet
+interface-name=eno1
+slave-type=bond
+master=bn0
+
+[ethernet]
+wake-on-lan=0
+
+[ipv4]
+method=link-local
+
+[ipv6]
+method=ignore
+''',
+                        'switchport': '''[connection]
+id=netplan-switchport
+type=ethernet
+interface-name=enp2s1
+slave-type=bond
+master=bn0
+
+[ethernet]
+wake-on-lan=0
+
+[ipv4]
+method=link-local
+
+[ipv6]
+method=ignore
+''',
+                        'bn0': '''[connection]
+id=netplan-bn0
+type=bond
+interface-name=bn0
+
+[bond]
+mode=active-backup
+primary=eno1
+
+[ipv4]
+method=auto
+
+[ipv6]
+method=ignore
 '''})
         self.assert_networkd({})
         self.assert_udev(None)
@@ -2441,6 +2840,9 @@ wake-on-lan=0
 method=manual
 address1=192.168.1.3/24
 dns=8.8.8.8;
+
+[ipv6]
+method=ignore
 '''})
 
     def test_vlan(self):
@@ -2467,6 +2869,9 @@ wake-on-lan=0
 
 [ipv4]
 method=link-local
+
+[ipv6]
+method=ignore
 ''',
                         'enblue': '''[connection]
 id=netplan-enblue
@@ -2480,6 +2885,9 @@ parent=en1
 [ipv4]
 method=manual
 address1=1.2.3.4/24
+
+[ipv6]
+method=ignore
 ''',
                         'engreen': '''[connection]
 id=netplan-engreen
@@ -2529,6 +2937,9 @@ mac-address=11:22:33:44:55:66
 
 [ipv4]
 method=link-local
+
+[ipv6]
+method=ignore
 ''' % uuid,
                         'engreen': '''[connection]
 id=netplan-engreen
@@ -2541,6 +2952,9 @@ parent=%s
 
 [ipv4]
 method=auto
+
+[ipv6]
+method=ignore
 ''' % uuid})
         self.assert_udev(None)
 
@@ -3054,6 +3468,38 @@ class TestConfigErrors(TestBase):
           - 2001:dead:beef::1
       dhcp4: true''', expect_fail=True)
 
+    def test_bond_invalid_primary_slave(self):
+        self.generate('''network:
+  version: 2
+  ethernets:
+    eno1:
+      match:
+        name: eth0
+  bonds:
+    bond0:
+      interfaces: [eno1]
+      parameters:
+        primary: wigglewiggle
+      dhcp4: true''', expect_fail=True)
+
+    def test_bond_duplicate_primary_slave(self):
+        self.generate('''network:
+  version: 2
+  ethernets:
+    eno1:
+      match:
+        name: eth0
+    eno2:
+      match:
+        name: eth1
+  bonds:
+    bond0:
+      interfaces: [eno1, eno2]
+      parameters:
+        primary: eno1
+        primary: eno2
+      dhcp4: true''', expect_fail=True)
+
 
 class TestForwardDeclaration(TestBase):
 
