@@ -34,8 +34,10 @@ for program in ['wpa_supplicant', 'hostapd', 'dnsmasq']:
         sys.exit(0)
 
 nm_uses_dnsmasq = b'dns=dnsmasq' in subprocess.check_output(['NetworkManager', '--print-config'])
-with open('/etc/nsswitch.conf') as f:
-    nss_resolve = ' resolve' in f.read()
+
+
+def resolved_in_use():
+    return os.path.isfile('/run/systemd/resolve/resolv.conf')
 
 
 class NetworkTestBase(unittest.TestCase):
@@ -80,7 +82,8 @@ class NetworkTestBase(unittest.TestCase):
             pass
 
     def tearDown(self):
-        subprocess.call(['systemctl', 'stop', 'NetworkManager', 'systemd-networkd', 'netplan-wpa@*'])
+        subprocess.call(['systemctl', 'stop', 'NetworkManager', 'systemd-networkd', 'netplan-wpa@*',
+                                              'systemd-networkd.socket'])
         # NM has KillMode=process and leaks dhclient processes
         subprocess.call(['systemctl', 'kill', 'NetworkManager'])
         subprocess.call(['systemctl', 'reset-failed', 'NetworkManager', 'systemd-networkd'],
@@ -137,7 +140,7 @@ class NetworkTestBase(unittest.TestCase):
         klass.dev_w_client = devs[1]
 
         # don't let NM trample over our fake AP
-        os.makedirs('/run/NetworkManager/conf.d')
+        os.makedirs('/run/NetworkManager/conf.d', exist_ok=True)
         with open('/run/NetworkManager/conf.d/test-blacklist.conf', 'w') as f:
             f.write('[main]\nplugins=keyfile\n[keyfile]\nunmanaged-devices+=nptestsrv,%s\n' % klass.dev_w_ap)
         # work around https://launchpad.net/bugs/1615044
@@ -476,7 +479,6 @@ class _CommonTests:
         with open('/sys/class/net/mybr/brif/%s/path_cost' % self.dev_e2_client) as f:
             self.assertEqual(f.read().strip(), '50')
 
-    @unittest.skip("SKIP on 16.10: systemd does not support bridge Priority yet")
     def test_bridge_priority(self):
         self.setup_eth(None)
         self.start_dnsmasq(None, self.dev_e2_ap)
@@ -509,7 +511,6 @@ class _CommonTests:
         with open('/sys/class/net/mybr/bridge/priority') as f:
             self.assertEqual(f.read().strip(), '8192')
 
-    @unittest.skip("SKIP on 16.10: systemd does not support bridge AgeingTime yet")
     def test_bridge_ageing_time(self):
         self.setup_eth(None)
         self.start_dnsmasq(None, self.dev_e2_ap)
@@ -694,6 +695,35 @@ class _CommonTests:
         with open('/sys/class/net/mybond/bonding/slaves') as f:
             self.assertEqual(f.read().strip(), self.dev_e_client)
 
+    def test_bond_primary_slave(self):
+        self.setup_eth(None)
+        self.addCleanup(subprocess.call, ['ip', 'link', 'delete', 'mybond'], stderr=subprocess.DEVNULL)
+        with open(self.config, 'w') as f:
+            f.write('''network:
+  renderer: %(r)s
+  ethernets:
+    %(ec)s: {}
+    %(e2c)s: {}
+  bonds:
+    mybond:
+      interfaces: [%(ec)s, %(e2c)s]
+      parameters:
+        mode: active-backup
+        primary: %(ec)s
+      addresses: [ '10.10.10.1/24' ]''' % {'r': self.backend, 'ec': self.dev_e_client, 'e2c': self.dev_e2_client})
+        self.generate_and_settle()
+        self.assert_iface_up(self.dev_e_client,
+                             ['master mybond'],
+                             ['inet '])
+        self.assert_iface_up('mybond',
+                             ['inet 10.10.10.1/24'])
+        with open('/sys/class/net/mybond/bonding/slaves') as f:
+            result = f.read().strip()
+            self.assertIn(self.dev_e_client, result)
+            self.assertIn(self.dev_e2_client, result)
+        with open('/sys/class/net/mybond/bonding/primary') as f:
+            self.assertEqual(f.read().strip(), '%(ec)s' % {'ec': self.dev_e_client})
+
     def test_bond_all_slaves_active(self):
         self.setup_eth(None)
         self.addCleanup(subprocess.call, ['ip', 'link', 'delete', 'mybond'], stderr=subprocess.DEVNULL)
@@ -996,6 +1026,31 @@ class _CommonTests:
         self.assertIn(b'metric 99',  # check metric from static route
                       subprocess.check_output(['ip', 'route', 'show', '10.10.10.0/24']))
 
+    def test_routes_v6(self):
+        self.setup_eth(None)
+        with open(self.config, 'w') as f:
+            f.write('''network:
+  renderer: %(r)s
+  ethernets:
+    %(ec)s:
+      addresses: ["9876:BBBB::11/70"]
+      gateway6: "9876:BBBB::1"
+      routes:
+          - to: 2001:f00f:f00f::1/64
+            via: 9876:BBBB::5
+            metric: 799''' % {'r': self.backend, 'ec': self.dev_e_client})
+        self.generate_and_settle()
+        self.assert_iface_up(self.dev_e_client,
+                             ['inet6 9876:bbbb::11/70'])
+        self.assertNotIn(b'default',
+                         subprocess.check_output(['ip', 'route', 'show', 'dev', self.dev_e_client]))
+        self.assertIn(b'default via 9876:bbbb::1',
+                      subprocess.check_output(['ip', '-6', 'route', 'show', 'dev', self.dev_e_client]))
+        self.assertIn(b'2001:f00f:f00f::/64 via 9876:bbbb::5',
+                      subprocess.check_output(['ip', '-6', 'route', 'show', 'dev', self.dev_e_client]))
+        self.assertIn(b'metric 799',
+                      subprocess.check_output(['ip', '-6', 'route', 'show', '2001:f00f:f00f::/64']))
+
     def test_manual_addresses(self):
         self.setup_eth(None)
         with open(self.config, 'w') as f:
@@ -1046,8 +1101,8 @@ class _CommonTests:
             out = subprocess.check_output(['journalctl', '--quiet', '-tdnsmasq', '-ocat', '--since=-30s'],
                                           universal_newlines=True)
             self.assertIn('nameserver 172.1.2.3', out)
-        elif nss_resolve:
-            sys.stdout.write('[nss-resolve] ')
+        elif resolved_in_use():
+            sys.stdout.write('[resolved] ')
             sys.stdout.flush()
             out = subprocess.check_output(['systemd-resolve', '--status'], universal_newlines=True)
             self.assertIn('DNS Servers: 172.1.2.3', out)
@@ -1247,12 +1302,12 @@ wpa_passphrase=12345678
   bridges:
     br0:
       interfaces: [bond0]
-      dhcp4: true
+      addresses: ['192.168.0.2/24']
   bonds:
     bond0:
       interfaces: [ethbn, ethb2]
       parameters:
-        mii-monitor-interval: 100000
+        mode: balance-rr
   ethernets:
     ethbn:
       match: {name: %(ec)s}
@@ -1268,8 +1323,9 @@ wpa_passphrase=12345678
                              ['inet '])
         self.assert_iface_up('bond0',
                              ['master br0'])
-        self.assert_iface_up('br0',
-                             ['inet 192.168'])
+        ipaddr = subprocess.check_output(['ip', 'a', 'show', 'dev', 'br0'],
+                                         universal_newlines=True)
+        self.assertIn('inet 192.168', ipaddr)
         with open('/sys/class/net/bond0/bonding/slaves') as f:
             result = f.read().strip()
             self.assertIn(self.dev_e_client, result)
@@ -1301,7 +1357,7 @@ wpa_passphrase=12345678
     bond0:
       interfaces: ['br1']
       parameters:
-        mii-monitor-interval: 100000
+        mode: balance-rr
   bridges:
     br1:
       interfaces: ['ethb2']
@@ -1329,6 +1385,35 @@ wpa_passphrase=12345678
 class TestNetworkd(NetworkTestBase, _CommonTests):
     backend = 'networkd'
 
+    def test_eth_dhcp6_off(self):
+        self.setup_eth('slaac')
+        with open(self.config, 'w') as f:
+            f.write('''network:
+  version: 2
+  renderer: %(r)s
+  ethernets:
+    %(ec)s:
+      dhcp6: no
+      addresses: [ '192.168.1.100/24' ]
+    %(e2c)s: {}''' % {'r': self.backend, 'ec': self.dev_e_client, 'e2c': self.dev_e2_client})
+        self.generate_and_settle()
+        self.assert_iface_up(self.dev_e_client, ['inet6 2600:'], [])
+
+    def test_eth_dhcp6_off_no_accept_ra(self):
+        self.setup_eth('slaac')
+        with open(self.config, 'w') as f:
+            f.write('''network:
+  version: 2
+  renderer: %(r)s
+  ethernets:
+    %(ec)s:
+      dhcp6: no
+      accept-ra: no
+      addresses: [ '192.168.1.100/24' ]
+    %(e2c)s: {}''' % {'r': self.backend, 'ec': self.dev_e_client, 'e2c': self.dev_e2_client})
+        self.generate_and_settle()
+        self.assert_iface_up(self.dev_e_client, [], ['inet6 2600:'])
+
     def test_bond_mac(self):
         self.setup_eth(None)
         self.addCleanup(subprocess.call, ['ip', 'link', 'delete', 'mybond'], stderr=subprocess.DEVNULL)
@@ -1356,6 +1441,7 @@ class TestNetworkd(NetworkTestBase, _CommonTests):
 
     def test_bridge_mac(self):
         self.setup_eth(None)
+        self.addCleanup(subprocess.call, ['ip', 'link', 'delete', 'br0'], stderr=subprocess.DEVNULL)
         self.start_dnsmasq(None, self.dev_e2_ap)
         with open(self.config, 'w') as f:
             f.write('''network:
@@ -1560,35 +1646,25 @@ class TestNetworkd(NetworkTestBase, _CommonTests):
         with open('/sys/class/net/mybond/bonding/arp_validate') as f:
             self.assertEqual(f.read().strip(), 'all 3')
 
-    def test_routes_v6(self):
-        self.setup_eth(None)
-        with open(self.config, 'w') as f:
-            f.write('''network:
-  renderer: %(r)s
-  ethernets:
-    %(ec)s:
-      addresses: ["9876:BBBB::11/70"]
-      gateway6: "9876:BBBB::1"
-      routes:
-          - to: 2001:f00f:f00f::1/64
-            via: 9876:BBBB::5
-            metric: 799''' % {'r': self.backend, 'ec': self.dev_e_client})
-        self.generate_and_settle()
-        self.assert_iface_up(self.dev_e_client,
-                             ['inet6 9876:bbbb::11/70'])
-        self.assertNotIn(b'default',
-                         subprocess.check_output(['ip', 'route', 'show', 'dev', self.dev_e_client]))
-        self.assertIn(b'default via 9876:bbbb::1',
-                      subprocess.check_output(['ip', '-6', 'route', 'show', 'dev', self.dev_e_client]))
-        self.assertIn(b'2001:f00f:f00f::/64 via 9876:bbbb::5',
-                      subprocess.check_output(['ip', '-6', 'route', 'show', 'dev', self.dev_e_client]))
-        self.assertIn(b'metric 799',
-                      subprocess.check_output(['ip', '-6', 'route', 'show', '2001:f00f:f00f::/64']))
-
 
 class TestNetworkManager(NetworkTestBase, _CommonTests):
     backend = 'NetworkManager'
 
+    @unittest.skip("NetworkManager does not disable accept_ra: bug LP: #1704210")
+    def test_eth_dhcp6_off(self):
+        self.setup_eth('slaac')
+        with open(self.config, 'w') as f:
+            f.write('''network:
+  version: 2
+  renderer: %(r)s
+  ethernets:
+    %(ec)s:
+      dhcp6: no
+      addresses: [ '192.168.1.100/24' ]
+    %(e2c)s: {}''' % {'r': self.backend, 'ec': self.dev_e_client, 'e2c': self.dev_e2_client})
+        self.generate_and_settle()
+        self.assert_iface_up(self.dev_e_client, [], ['inet6 2600:'])
+
     @unittest.skip("NetworkManager does not support setting MAC for a bond")
     def test_bond_mac(self):
         pass
@@ -1753,7 +1829,6 @@ class TestNetworkManager(NetworkTestBase, _CommonTests):
         with open('/sys/class/net/mybond/bonding/arp_ip_target') as f:
             self.assertEqual(f.read().strip(), '192.168.5.1')
 
-    @unittest.skip("SKIP 16.10: broken on 16.10 versions of linux/NetworkManager")
     def test_bond_arp_all_targets(self):
         self.setup_eth(None)
         self.start_dnsmasq(None, self.dev_e2_ap)
@@ -1817,9 +1892,5 @@ class TestNetworkManager(NetworkTestBase, _CommonTests):
         with open('/sys/class/net/mybond/bonding/lp_interval') as f:
             self.assertEqual(f.read().strip(), '15')
 
-    @unittest.skip("SKIP 16.10: broken on 16.10 versions of linux/NetworkManager")
-    def test_routes_v6(self):
-        pass
-
 
 unittest.main(testRunner=unittest.TextTestRunner(stream=sys.stdout, verbosity=2))
